/*
Copyright 2022.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package v1

import (
	corev1 "k8s.io/api/core/v1"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
)

// BlueGreenSpec defines the desired state of BlueGreen
type BlueGreenSpec struct {
	// Set which service traffic should go
	RouteTo BlueOrGreen `json:"routeTo"`

	// Pod Spec for a Blue Service
	BlueSpec *corev1.PodSpec `json:"blueSpec,omitempty"`

	// Pod Spec for a Green Service
	GreenSpec *corev1.PodSpec `json:"greenSpec,omitempty"`
}

// +kubebuilder:validation:Enum=Blue;Green
type BlueOrGreen string

const (
	Blue  BlueOrGreen = "Blue"
	Green BlueOrGreen = "Green"
)

// BlueGreenStatus defines the observed state of BlueGreen
type BlueGreenStatus struct {
	RouteTo *BlueOrGreen `json:"routeTo,omitempty"`
}

//+kubebuilder:object:root=true
//+kubebuilder:subresource:status
//+kubebuilder:printcolumn:name="RouteTo",type=string,JSONPath=`.status.routeTo`
//+kubebuilder:printcolumn:name="BlueReady",type=integer,JSONPath=`.status.blueReady`
//+kubebuilder:printcolumn:name="GreenReady",type=integer,JSONPath=`.status.greenReady`

// BlueGreen is the Schema for the bluegreens API
type BlueGreen struct {
	metav1.TypeMeta   `json:",inline"`
	metav1.ObjectMeta `json:"metadata,omitempty"`

	Spec   BlueGreenSpec   `json:"spec,omitempty"`
	Status BlueGreenStatus `json:"status,omitempty"`
}

//+kubebuilder:object:root=true

// BlueGreenList contains a list of BlueGreen
type BlueGreenList struct {
	metav1.TypeMeta `json:",inline"`
	metav1.ListMeta `json:"metadata,omitempty"`
	Items           []BlueGreen `json:"items"`
}

func init() {
	SchemeBuilder.Register(&BlueGreen{}, &BlueGreenList{})
}
